@@ -1,4 +1,4 @@
-/* $Id: xwit.c,v 3.3 95/10/18 16:59:18 dd Exp $ */
+/* $Id: xwit.c,v 3.4 97/10/20 18:32:49 dd Exp $ */
 /*
  * Pop up or iconify the current xterm window (using its WINDOW_ID in the env)
  * or a given window id or a list of window matching names. etc...
@@ -37,7 +37,7 @@
 /* note: called by dsimple.c code, must be global */
 usage()
 {
-	static char Revision[] = "$Revision: 3.3 $";
+	static char Revision[] = "$Revision: 3.4 $";
 	char *rbeg, *rend;
 
 	for (rbeg = Revision; *rbeg; rbeg++) {
@@ -65,7 +65,7 @@ usage()
 	-bitmap <file> -mask <file> -[r]iconmove x y\n\
 	-[no]backingstore -[no]saveunder\n\
 	-[no]keyrepeat keycode ... keycode - keycode\n\
-	-id <windowid> -root -select\n\
+	-id <windowid> -root -current -select\n\
 	-names <initialsubstrings>... [must be last]\n",
 		program_name);
 	exit(2);
@@ -97,6 +97,7 @@ static enum winidmode {
 	WID_num,
 	WID_root,
 	WID_select,
+	WID_curr,
 	WID_names,
 } Winidmode;
 
@@ -256,6 +257,13 @@ domove(window, x, y, right, bottom)
 			if (status && childlist)
 				XFree((char *) childlist);
 		}
+
+#ifndef FVWM2
+		/*
+		 * Norman R. McBride <norm@city.ac.uk> reports that
+		 * this code doesn't work with fvwm2, and I don't have
+		 * a fix yet, sorry. - DD
+		 */
 		if (wmframe != window) {
 			if (!XGetWindowAttributes(dpy, wmframe, &frame_attr))
 				Fatal_Error("XGetWindowAttributes(0x%x)",
@@ -266,6 +274,7 @@ domove(window, x, y, right, bottom)
 			win_attr.border_width +=
 				frame_attr.border_width;
 		}
+#endif /* !FVWM2 */
 
 		if (right)
 			x += DisplayWidth(dpy, screen) -
@@ -580,21 +589,29 @@ doit(window)
 	}
 }
 
-/* this code stolen from xwininfo.c */
+/* based on xwininfo.c */
 static Window
-xwit_select_window(dpy)
+xwit_select_window(dpy, current)
 	Display *dpy;
+	int current;
 {
-	Window window;
+	Window window = None;
 	Window wroot;
 	int dummyi;
 	unsigned int dummy;
 
-	printf("\n");
-	printf("%s: select window by clicking the mouse\n",
-		program_name);
-	(void) fflush(stdout);
-	window = Select_Window(dpy);
+	if (current) {
+		XQueryPointer(dpy, RootWindow(dpy, screen),
+			&wroot, &window,
+			&dummyi, &dummyi, &dummyi, &dummyi, &dummy);
+	}
+	else {
+		printf("\n");
+		printf("%s: select window by clicking the mouse\n",
+			program_name);
+		(void) fflush(stdout);
+		window = Select_Window(dpy);
+	}
 	if (window) {
 		if (XGetGeometry(dpy, window, &wroot, &dummyi, &dummyi,
 			&dummy, &dummy, &dummy, &dummy) &&
@@ -619,25 +636,71 @@ getxid(s)
 }
 
 static int
-matchopt(key, len, nargs, argc, argv)
+matchopt(key, nargs, argc, argv)
 	char *key;
-	int len;
 	int nargs;
 	int *argc;
 	char **argv;
 {
-	int match;
+	int enough = 0;
+	int match = 1;
+	char *ap, *kp;
 
-	if (len)
-		match = strncmp(key, *argv, len) == 0;
-	else
-		match = strcmp(key, *argv) == 0;
+	ap = *argv;
+	if (*ap == '-')
+		ap++;
+
+	for (kp = key; *kp; kp++, ap++) {
+		if (*kp == '*') {
+			enough = 1;
+			ap--;
+			continue;
+		}
+		if (*ap == 0) {
+			match = enough;
+			break;
+		}
+		if (*ap != *kp) {
+			match = 0;
+			break;
+		}
+	}
 
 	if (match) {
 		if (argc[0] <= nargs) {
+			char option[32];
+			int dash, skip;
+
+			strncpy(option, *argv, sizeof option - 1);
+			option[sizeof option - 1] = 0;
+
+			dash = *argv[0] == '-';
+
+			for (ap = option; *ap; ap++)
+				/* nothing */ ;
+			skip = ap - option - dash;
+			enough = 0;
+	
+			for (kp = key; *kp && skip--; kp++) {
+				if (*kp == '*')
+					skip++;
+			}
+			for (; *kp; kp++) {
+				if (*kp == '*')
+					continue;
+				if (enough == 0) {
+					*ap++ = '(';
+					enough = 1;
+				}
+				*ap++ = *kp;
+			}
+			if (enough)
+				*ap++ = ')';
+			*ap = 0;
+
 			fprintf(stderr,
 				"%s: option %s needs %d argument%s\n\n",
-				program_name, key,
+				program_name, option,
 				nargs, nargs > 1 ? "s" : "");
 			usage();
 		}
@@ -646,226 +709,252 @@ matchopt(key, len, nargs, argc, argv)
 	return match;
 }
 
-
-main(argc,argv)
-    int argc;
-    char **argv;
+main(argc, argv)
+	int argc;
+	char **argv;
 {
-    Window window = 0;
-    int *pargc = &argc;
+	Window window = 0;
+	int *pargc = &argc;
 
-    program_name = argv[0] + strlen(argv[0]);
-    while (program_name != argv[0] && program_name[-1] != '/')
-	program_name--;
+	program_name = argv[0] + strlen(argv[0]);
+	while (program_name != argv[0] && program_name[-1] != '/')
+		program_name--;
 
-    Setup_Display_And_Screen(pargc, argv);
+	Setup_Display_And_Screen(pargc, argv);
 
-    Winidmode = WID_env;
+	Winidmode = WID_env;
 
-    while (argv++, --argc > 0) {
-	/* argv[0] = next argument */
-	/* argc = # of arguments left */
-	int argvlen = strlen(*argv);
-	if(argvlen<2)argvlen = 2;
-	if (matchopt("-id", 0, 1, pargc, argv)) {
-	    Winidmode = WID_num;
-	    window = (Window) getxid(*++argv);
-	}
-	else if (matchopt("-root", 0, 0, pargc, argv)) {
-		Winidmode = WID_root;
-	}
-	else if (matchopt("-names", 6, 1, pargc, argv)) {
-		Winidmode = WID_names;
-		/* take rest of arg list */
-		names = ++argv;
-		numnames = argc;
-		argc = 0;
-	}
-	else if (matchopt("-select", argvlen, 0, pargc, argv)) {
-		Winidmode = WID_select;
-	}
-	else if (matchopt("-colormap", argvlen, 1, pargc, argv) ||
-	    matchopt("-cmap", 0, 1, pargc, argv)) {
-	    function |= FBIT(colormap);
-	    cmap = (Colormap) getxid(*++argv);
-	}
-	else if (matchopt("-move", 0, 2, pargc, argv)) {
-		function |= FBIT(move);
-		Gright = (argv[1][0] == '-');
-		Gbottom = (argv[2][0] == '-');
-		tox = atoi(argv[1]);
-		toy = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-rmove", 0, 2, pargc, argv)) {
-		function |= FBIT(rmove);
-		tox = atoi(argv[1]);
-		toy = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-resize", argvlen, 2, pargc, argv)) {
-		function |= FBIT(resize);
-		towidth = atoi(argv[1]);
-		toheight = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-warp", 0, 2, pargc, argv) ||
-		matchopt("-rwarp", 0, 2, pargc, argv)) {
-		function |= argv[0][1] == 'r' ? FBIT(rwarp) : FBIT(warp);
-		warpx = atoi(argv[1]);
-		warpy = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-pop", 0, 0, pargc, argv)) {
-		function |= FBIT(pop);
-	}
-	else if (matchopt("-save", 0, 0, pargc, argv)) {
-		function |= FBIT(save);
-	}
-	else if (matchopt("-nosave", 0, 0, pargc, argv)) {
-		function |= FBIT(nosave);
-	}
-	else if (matchopt("-iconify", argvlen, 0, pargc, argv)) {
-		function |= FBIT(icon);
-	}
-	else if (matchopt("-unmap", argvlen, 0, pargc, argv)) {
-		function |= FBIT(unmap);
-	}
-	else if (matchopt("-keyrepeat", argvlen, 1, pargc, argv) ||
-		matchopt("-nokeyrepeat", 3, 1, pargc, argv)) {
-		int i;
+	while (argv++, --argc > 0) {
+		/* argv[0] = next argument */
+		/* argc = # of arguments left */
+		if (matchopt("ba*ckingstore", 0, pargc, argv) ||
+			matchopt("bs", 0, pargc, argv)) {
+			function |= FBIT(F_winattr);
+			Gbs = 1;
+		}
+		else if (matchopt("b*itmap", 1, pargc, argv)) {
+			function |= FBIT(iconbitmap);
+			bitmapname = *++argv;
+		}
+		else if (matchopt("colo*rmap", 1, pargc, argv) ||
+			matchopt("cm*ap", 1, pargc, argv)) {
+			function |= FBIT(colormap);
+			cmap = (Colormap) getxid(*++argv);
+		}
+		else if (matchopt("co*lumns", 1, pargc, argv)) {
+			function |= FBIT(columns);
+			ncolumns = atoi(*++argv);
+		}
+		else if (matchopt("c*urrent", 0, pargc, argv)) {
+			Winidmode = WID_curr;
+		}
+		else if (matchopt("iconm*ove", 2, pargc, argv)) {
+			function |= FBIT(iconmove);
+			Giconx = atoi(argv[1]);
+			Gicony = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("iconn*ame", 1, pargc, argv) ||
+			matchopt("in*ame", 1, pargc, argv)) {
+			function |= FBIT(iconname);
+			wmiconname = *++argv;
+		}
+		else if (matchopt("id", 1, pargc, argv)) {
+			Winidmode = WID_num;
+			window = (Window) getxid(*++argv);
+		}
+		else if (matchopt("i*conify", 0, pargc, argv)) {
+			function |= FBIT(icon);
+		}
+		else if (matchopt("k*eyrepeat", 1, pargc, argv) ||
+			matchopt("nok*eyrepeat", 1, pargc, argv)) {
+			int i;
 
-		function |= argv[0][1] == 'n' ?
-			FBIT(nokeyrepeat) : FBIT(keyrepeat);
+			function |= argv[0][1] == 'n' ?
+				FBIT(nokeyrepeat) : FBIT(keyrepeat);
 
-		i = atoi(*++argv);
-		if (i < 0)
-			usage();
+			i = atoi(*++argv);
+			if (i < 0)
+				usage();
 
-		while (1) {
-			keys[i & 0xff] = 1;
-			if (argc <= 0)
-				break;
-			if (strcmp(argv[0], "-") == 0) {
-				int from = i;
-
-				argc--, argv++;
-				if (argc < 0 || (i = atoi(argv[0])) <= 0)
-					usage();
-				while (from <= i)
-					keys[from++ & 0xff] = 1;
-				argc--, argv++;
+			while (1) {
+				keys[i & 0xff] = 1;
 				if (argc <= 0)
 					break;
+				if (strcmp(argv[0], "-") == 0) {
+					int from = i;
+
+					argc--, argv++;
+					if (argc < 0 || (i = atoi(argv[0])) <= 0)
+						usage();
+					while (from <= i)
+						keys[from++ & 0xff] = 1;
+					argc--, argv++;
+					if (argc <= 0)
+						break;
+				}
+				if ((i = atoi(argv[0])) <= 0)
+					break;
+				argc--, argv++;
 			}
-			if ((i = atoi(argv[0])) <= 0)
-				break;
-			argc--, argv++;
 		}
+		else if (matchopt("li*st", 1, pargc, argv) ||
+			matchopt("names", 1, pargc, argv)) {
+			Winidmode = WID_names;
+			/* take rest of arg list */
+			names = ++argv;
+			numnames = argc;
+			argc = 0;
+		}
+		else if (matchopt("l*abel", 1, pargc, argv)) {
+			function |= FBIT(name);
+			wmname = *++argv;
+		}
+		else if (matchopt("ma*sk", 1, pargc, argv)) {
+			function |= FBIT(iconbitmap);
+			maskname = *++argv;
+		}
+		else if (matchopt("m*ove", 2, pargc, argv)) {
+			function |= FBIT(move);
+			Gright = (argv[1][0] == '-');
+			Gbottom = (argv[2][0] == '-');
+			tox = atoi(argv[1]);
+			toy = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("noba*ckingstore", 0, pargc, argv) ||
+			matchopt("nobs", 0, pargc, argv)) {
+			function |= FBIT(F_winattr);
+			Gbs = -1;
+		}
+		else if (matchopt("nosaveu*nder", 0, pargc, argv) ||
+			matchopt("nosu", 0, pargc, argv)) {
+			function |= FBIT(F_winattr);
+			Gsu = -1;
+		}
+		else if (matchopt("nos*ave", 0, pargc, argv)) {
+			function |= FBIT(nosave);
+		}
+		else if (matchopt("n*ame", 1, pargc, argv)) {
+			function |= FBIT(name);
+			wmname = *++argv;
+		}
+		else if (matchopt("o*pen", 0, pargc, argv)) {
+			function |= FBIT(pop);
+		}
+		else if (matchopt("p*op", 0, pargc, argv)) {
+			function |= FBIT(pop);
+		}
+		else if (matchopt("ri*conmove", 2, pargc, argv)) {
+			function |= FBIT(riconmove);
+			Giconx = atoi(argv[1]);
+			Gicony = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("rm*ove", 2, pargc, argv)) {
+			function |= FBIT(rmove);
+			tox = atoi(argv[1]);
+			toy = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("roo*t", 0, pargc, argv)) {
+			Winidmode = WID_root;
+		}
+		else if (matchopt("ro*ws", 1, pargc, argv)) {
+			function |= FBIT(rows);
+			nrows = atoi(*++argv);
+		}
+		else if (matchopt("rw*arp", 2, pargc, argv)) {
+			function |= FBIT(rwarp);
+			warpx = atoi(argv[1]);
+			warpy = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("r*esize", 2, pargc, argv)) {
+			function |= FBIT(resize);
+			towidth = atoi(argv[1]);
+			toheight = atoi(argv[2]);
+			argv += 2;
+		}
+		else if (matchopt("saveu*nder", 0, pargc, argv) ||
+			matchopt("su", 0, pargc, argv)) {
+			function |= FBIT(F_winattr);
+			Gsu = argv[0][1] == 1;
+		}
+		else if (matchopt("se*lect", 0, pargc, argv)) {
+			Winidmode = WID_select;
+		}
+		else if (matchopt("sy*nc", 0, pargc, argv)) {
+			XSynchronize(dpy, True);
+		}
+		else if (matchopt("s*ave", 0, pargc, argv)) {
+			function |= FBIT(save);
+		}
+		else if (matchopt("un*map", 0, pargc, argv)) {
+			function |= FBIT(unmap);
+		}
+		else if (matchopt("w*arp", 2, pargc, argv)) {
+			function |= FBIT(warp);
+			warpx = atoi(argv[1]);
+			warpy = atoi(argv[2]);
+			argv += 2;
+		}
+		else
+			usage();
 	}
-	else if (matchopt("-name", 0, 1, pargc, argv) ||
-		matchopt("-label", argvlen, 1, pargc, argv)) {
-		function |= FBIT(name);
-		wmname = *++argv;
-	}
-	else if (matchopt("-iconname", 0, 1, pargc, argv)) {
-		function |= FBIT(iconname);
-		wmiconname = *++argv;
-	}
-	else if (matchopt("-iconmove", 0, 2, pargc, argv)) {
-		function |= FBIT(iconmove);
-		Giconx = atoi(argv[1]);
-		Gicony = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-riconmove", 0, 2, pargc, argv)) {
-		function |= FBIT(riconmove);
-		Giconx = atoi(argv[1]);
-		Gicony = atoi(argv[2]);
-		argv += 2;
-	}
-	else if (matchopt("-rows", argvlen, 1, pargc, argv)) {
-		function |= FBIT(rows);
-		nrows = atoi(*++argv);
-	}
-	else if (matchopt("-columns", argvlen, 1, pargc, argv)) {
-		function |= FBIT(columns);
-		ncolumns = atoi(*++argv);
-	}
-	else if (matchopt("-bitmap", 0, 1, pargc, argv)) {
-		function |= FBIT(iconbitmap);
-		bitmapname = *++argv;
-	}
-	else if (matchopt("-mask", 0, 1, pargc, argv)) {
-		function |= FBIT(iconbitmap);
-		maskname = *++argv;
-	}
-	else if (matchopt("-backingstore", argvlen, 0, pargc, argv) ||
-		matchopt("-nobackingstore", 3, 0, pargc, argv) ||
-		matchopt("-bs", 0, 0, pargc, argv) ||
-		matchopt("-nobs", 0, 0, pargc, argv)) {
-		function |= FBIT(F_winattr);
-		Gbs = argv[0][1] == 'n' ? -1 : 1;
-	}
-	else if (matchopt("-saveunder", argvlen, 0, pargc, argv) ||
-		matchopt("-nosaveunder", 3, 0, pargc, argv) ||
-		matchopt("-su", 0, 0, pargc, argv) ||
-		matchopt("-nosu", 0, 0, pargc, argv)) {
-		function |= FBIT(F_winattr);
-		Gsu = argv[0][1] == 'n' ? -1 : 1;
-	}
-	else if (matchopt("-sync", 0, 0, pargc, argv)) {
-		XSynchronize(dpy, True);
-	}
-	else
-		usage();
-    }
 
-    /* default function: pop */
-    if (function == 0)
-	function = FBIT(pop);
+	/* default function: pop */
+	if (function == 0)
+		function = FBIT(pop);
 
-    if ((function & ~NOWINDOW) == 0)
-	Winidmode = WID_none;
+	if ((function & ~NOWINDOW) == 0)
+		Winidmode = WID_none;
 
-    root = DefaultRootWindow(dpy);
+	root = DefaultRootWindow(dpy);
 
-    switch (Winidmode) {
-    case WID_env:
-	{
-	    char *s;
-	    extern char *getenv();
+	switch (Winidmode) {
+	case WID_env:
+		{
+			char *s;
+			extern char *getenv();
 
-	    s = getenv("WINDOWID");
-	    if (s != 0)
-		window = (Window) getxid(s);
-	    else
-		Fatal_Error("WINDOWID not set");
+			s = getenv("WINDOWID");
+			if (s != 0)
+				window = (Window) getxid(s);
+			else {
+				/* no WINDOWID, use window under cursor */
+				window = xwit_select_window(dpy, 1);
+				if (window == None)
+					Fatal_Error("WINDOWID not set");
+			}
+		}
+		break;
+	case WID_root:
+		window = root;
+		break;
+	case WID_curr:
+		window = xwit_select_window(dpy, 1);
+		break;
+	case WID_select:
+		window = xwit_select_window(dpy, 0);
+		break;
 	}
-	break;
-    case WID_root:
-	window = root;
-	break;
-    case WID_select:
-	window = xwit_select_window(dpy);
-	break;
-    }
 
-    switch (Winidmode) {
-    case WID_none:
-	doit((Window) 0);
-	break;
-    case WID_names:
-	downtree(root);
-	break;
-    default:
-	if (!window)
-	    Fatal_Error("no window selected");
-	doit(window);
-	break;
-    }
+	switch (Winidmode) {
+	case WID_none:
+		doit((Window) 0);
+		break;
+	case WID_names:
+		downtree(root);
+		break;
+	default:
+		if (!window)
+			Fatal_Error("no window selected");
+		doit(window);
+		break;
+	}
 
-    XSync(dpy,True);
-    (void) XCloseDisplay(dpy);
-    exit(!Gwinfound);
+	XSync(dpy, True);
+	(void) XCloseDisplay(dpy);
+	exit(!Gwinfound);
 }
